@@ -20,6 +20,9 @@ const DND = imports.ui.dnd;
 const Meta = imports.gi.Meta;
 const DocInfo = imports.misc.docInfo;
 const GLib = imports.gi.GLib;
+const Settings = imports.ui.settings;
+const Pango = imports.gi.Pango;
+
 const ICON_SIZE = 16;
 const MAX_FAV_ICON_SIZE = 32;
 const CATEGORY_ICON_SIZE = 22;
@@ -73,14 +76,14 @@ VisibleChildIterator.prototype = {
 
     getNextVisible: function(cur_child) {
         if (this.visible_children.indexOf(cur_child) == this._num_children-1)
-            return cur_child;
+            return this.visible_children[0];
         else
             return this.visible_children[this.visible_children.indexOf(cur_child)+1];
     },
 
     getPrevVisible: function(cur_child) {
         if (this.visible_children.indexOf(cur_child) == 0)
-            return cur_child;
+            return this.visible_children[this._num_children-1];
         else
             return this.visible_children[this.visible_children.indexOf(cur_child)-1];
     },
@@ -133,6 +136,13 @@ ApplicationContextMenuItem.prototype = {
                 let desktopFiles = settings.get_strv('panel-launchers');
                 desktopFiles.push(this._appButton.app.get_id());
                 settings.set_strv('panel-launchers', desktopFiles);
+                if (!Main.AppletManager.get_object_for_uuid("panel-launchers@cinnamon.org")){
+                    var new_applet_id = global.settings.get_int("next-applet-id");
+                    global.settings.set_int("next-applet-id", (new_applet_id + 1));
+                    var enabled_applets = global.settings.get_strv("enabled-applets");
+                    enabled_applets.push("panel1:right:0:panel-launchers@cinnamon.org:" + new_applet_id);
+                    global.settings.set_strv("enabled-applets", enabled_applets);
+                }
                 break;
             case "add_to_desktop":
                 let file = Gio.file_new_for_path(this._appButton.app.get_app_info().get_filename());
@@ -341,13 +351,23 @@ ApplicationButton.prototype = {
         GenericApplicationButton.prototype._init.call(this, appsMenuButton, app, true);
         this.category = new Array();
         this.actor.set_style_class_name('menu-application-button');
-        this.icon = this.app.create_icon_texture(APPLICATION_ICON_SIZE);
+
+        this.icon = null;                
+        if (this.app.get_app_info() != null && this.app.get_app_info().get_icon() != null) {
+            this.icon = new St.Icon({gicon: this.app.get_app_info().get_icon(), icon_size: APPLICATION_ICON_SIZE, icon_type: St.IconType.FULLCOLOR});
+        }
+        if (this.icon == null) {
+            this.icon = new St.Icon({icon_name: "application-x-executable", icon_size: APPLICATION_ICON_SIZE, icon_type: St.IconType.FULLCOLOR});        
+        }
+
         this.addActor(this.icon);
         this.name = this.app.get_name();
         this.label = new St.Label({ text: this.name, style_class: 'menu-application-button-label' });
         this.addActor(this.label);
         this._draggable = DND.makeDraggable(this.actor);
         this.isDraggableApp = true;
+        this.icon.realize();
+        this.label.realize();
     },
     
     get_app_id: function() {
@@ -387,8 +407,13 @@ PlaceButton.prototype = {
         this.actor._delegate = this;
         this.label = new St.Label({ text: this.button_name, style_class: 'menu-application-button-label' });
         this.icon = place.iconFactory(APPLICATION_ICON_SIZE);
-        this.addActor(this.icon);
+        if (!this.icon)
+            this.icon = new St.Icon({icon_name: "folder", icon_size: APPLICATION_ICON_SIZE, icon_type: St.IconType.FULLCOLOR});
+        if (this.icon)
+            this.addActor(this.icon);
         this.addActor(this.label);
+        this.icon.realize();
+        this.label.realize();
     },
 
     _onButtonReleaseEvent: function (actor, event) {
@@ -419,9 +444,13 @@ RecentButton.prototype = {
         this.actor.set_style_class_name('menu-application-button');
         this.actor._delegate = this;
         this.label = new St.Label({ text: this.button_name, style_class: 'menu-application-button-label' });
+        this.label.clutter_text.ellipsize = Pango.EllipsizeMode.END;        
+        this.label.set_style("max-width: 250px;");
         this.icon = file.createIcon(APPLICATION_ICON_SIZE);
         this.addActor(this.icon);
         this.addActor(this.label);
+        this.icon.realize();
+        this.label.realize();
     },
 
     _onButtonReleaseEvent: function (actor, event) {
@@ -492,8 +521,10 @@ CategoryButton.prototype = {
         if (category && this.icon_name) {
             this.icon = new St.Icon({icon_name: this.icon_name, icon_size: CATEGORY_ICON_SIZE, icon_type: St.IconType.FULLCOLOR});
             this.addActor(this.icon);
+            this.icon.realize();
         }
         this.addActor(this.label);
+        this.label.realize();
     }
 };
 
@@ -511,7 +542,9 @@ PlaceCategoryButton.prototype = {
         this.label = new St.Label({ text: _("Places"), style_class: 'menu-category-button-label' });
         this.icon = new St.Icon({icon_name: "folder", icon_size: CATEGORY_ICON_SIZE, icon_type: St.IconType.FULLCOLOR});
         this.addActor(this.icon);
+        this.icon.realize();
         this.addActor(this.label);
+        this.label.realize();
     }
 };
 
@@ -529,7 +562,9 @@ RecentCategoryButton.prototype = {
         this.label = new St.Label({ text: _("Recent Files"), style_class: 'menu-category-button-label' });
         this.icon = new St.Icon({icon_name: "folder-recent", icon_size: CATEGORY_ICON_SIZE, icon_type: St.IconType.FULLCOLOR});
         this.addActor(this.icon);
+        this.icon.realize()
         this.addActor(this.label);
+        this.label.realize();
     }
 };
 
@@ -548,9 +583,19 @@ FavoritesButton.prototype = {
         if (icon_size>MAX_FAV_ICON_SIZE) icon_size = MAX_FAV_ICON_SIZE;
         this.actor.style = "padding-top: "+(icon_size/3)+"px;padding-bottom: "+(icon_size/3)+"px; margin:auto;"
 
-        this.actor.add_style_class_name('menu-favorites-button');
-        this.addActor(app.create_icon_texture(icon_size));  
-        
+        this.actor.add_style_class_name('menu-favorites-button');    
+
+        let icon = null;                
+        if (app.get_app_info() != null && app.get_app_info().get_icon() != null) {
+            icon = new St.Icon({gicon: app.get_app_info().get_icon(), icon_size: icon_size, icon_type: St.IconType.FULLCOLOR});
+        }
+        if (icon == null) {
+            icon = new St.Icon({icon_name: "application-x-executable", icon_size: icon_size, icon_type: St.IconType.FULLCOLOR});        
+        }
+
+        this.addActor(icon);
+        icon.realize()
+
         this._draggable = DND.makeDraggable(this.actor);     
         this.isDraggableApp = true;
     },
@@ -583,7 +628,8 @@ SystemButton.prototype = {
         if (icon_size>MAX_FAV_ICON_SIZE) icon_size = MAX_FAV_ICON_SIZE;
         this.actor.style = "padding-top: "+(icon_size/3)+"px;padding-bottom: "+(icon_size/3)+"px; margin:auto;"
         let iconObj = new St.Icon({icon_name: icon, icon_size: icon_size, icon_type: St.IconType.FULLCOLOR});
-        this.actor.set_child(iconObj);             
+        this.actor.set_child(iconObj);
+        iconObj.realize()
     }
 };
 
@@ -754,15 +800,15 @@ FavoritesBox.prototype = {
     }
 }
 
-function MyApplet(orientation, panel_height) {
-    this._init(orientation, panel_height);
+function MyApplet(orientation, panel_height, instance_id) {
+    this._init(orientation, panel_height, instance_id);
 }
 
 MyApplet.prototype = {
     __proto__: Applet.TextIconApplet.prototype,
 
-    _init: function(orientation, panel_height) {        
-        Applet.TextIconApplet.prototype._init.call(this, orientation, panel_height);
+    _init: function(orientation, panel_height, instance_id) {        
+        Applet.TextIconApplet.prototype._init.call(this, orientation, panel_height, instance_id);
         
         try {                    
             this.set_applet_tooltip(_("Menu"));
@@ -772,46 +818,22 @@ MyApplet.prototype = {
             this.menuManager.addMenu(this.menu);   
                         
             this.actor.connect('key-press-event', Lang.bind(this, this._onSourceKeyPress));
-            this.showRecent = global.settings.get_boolean("menu-show-recent");
-            global.settings.connect("changed::menu-show-recent", Lang.bind(this, function() {
-                this.showRecent = global.settings.get_boolean("menu-show-recent");
-                this._refreshPlacesAndRecent();
-            }));
-            this.showPlaces = global.settings.get_boolean("menu-show-places");
-            global.settings.connect("changed::menu-show-places", Lang.bind(this, function() {
-                this.showPlaces = global.settings.get_boolean("menu-show-places");
-                this._refreshPlacesAndRecent();
-            }));
-            let updateActivateOnHover = Lang.bind(this, function() {
-                if (this._openMenuId) {
-                    this.actor.disconnect(this._openMenuId);
-                    this._openMenuId = 0;
-                }
-                let openOnHover = global.settings.get_boolean("activate-menu-applet-on-hover");
-                if (openOnHover) {
-                    this._openMenuId = this.actor.connect('enter-event', Lang.bind(this, this.openMenu));
-                }
-            });
-            updateActivateOnHover();
-            global.settings.connect("changed::activate-menu-applet-on-hover", updateActivateOnHover);
-                        
+
+            this.settings = new Settings.AppletSettings(this, "menu@cinnamon.org", instance_id);
+
+            this.settings.bindProperty(Settings.BindingDirection.IN, "show-recent", "showRecent", this._refreshPlacesAndRecent, null);
+            this.settings.bindProperty(Settings.BindingDirection.IN, "show-places", "showPlaces", this._refreshPlacesAndRecent, null);
+
+            this.settings.bindProperty(Settings.BindingDirection.IN, "activate-on-hover", "activateOnHover", this._updateActivateOnHover, null);
+            this._updateActivateOnHover();
+
             this.menu.actor.add_style_class_name('menu-background');
             this.menu.connect('open-state-changed', Lang.bind(this, this._onOpenStateChanged));                                
-                                    
-            this._updateIcon();
-            
-            global.settings.connect("changed::menu-icon", Lang.bind(this, function() {
-                this._updateIcon();
-            })); 
-            
-            this.set_applet_label(_("Menu"));                                            
-            let menuLabel = global.settings.get_string("menu-text");
-            if (menuLabel != "Menu") {
-                this.set_applet_label(menuLabel);                 
-            } 
-            global.settings.connect("changed::menu-text", Lang.bind(this, function() {
-                    this.set_applet_label(global.settings.get_string("menu-text"));
-                })); 
+
+            this.settings.bindProperty(Settings.BindingDirection.IN, "menu-icon", "menuIcon", this._updateIconAndLabel, null);
+            this.settings.bindProperty(Settings.BindingDirection.IN, "menu-label", "menuLabel", this._updateIconAndLabel, null);
+            this._updateIconAndLabel();
+
             this._searchInactiveIcon = new St.Icon({ style_class: 'menu-search-entry-icon',
                                                icon_name: 'edit-find',
                                                icon_type: St.IconType.SYMBOLIC });
@@ -820,7 +842,7 @@ MyApplet.prototype = {
                                              icon_type: St.IconType.SYMBOLIC });
             this._searchIconClickedId = 0;
             this._applicationsButtons = new Array();
-	    this._applicationsButtonFromApp = new Object();
+            this._applicationsButtonFromApp = new Object();
             this._favoritesButtons = new Array();
             this._placesButtons = new Array();
             this._transientButtons = new Array();
@@ -842,11 +864,9 @@ MyApplet.prototype = {
             appsys.connect('installed-changed', Lang.bind(this, this._refreshApps));
             AppFavorites.getAppFavorites().connect('changed', Lang.bind(this, this._refreshFavs));
 
-            this.hover_delay = global.settings.get_int("menu-hover-delay") / 1000;
-            global.settings.connect("changed::menu-hover-delay", Lang.bind(this, function() {
-                    this.hover_delay = global.settings.get_int("menu-hover-delay") / 1000;
-            })); 
-                
+            this.settings.bindProperty(Settings.BindingDirection.IN, "hover-delay", "hover_delay_ms", this._update_hover_delay, null);
+            this._update_hover_delay();
+
             global.display.connect('overlay-key', Lang.bind(this, function(){
                 try{
                     this.menu.toggle_with_options(false);
@@ -858,24 +878,13 @@ MyApplet.prototype = {
             Main.placesManager.connect('places-updated', Lang.bind(this, this._refreshPlacesAndRecent));
             this.RecentManager.connect('changed', Lang.bind(this, this._refreshPlacesAndRecent));
 
-            this.edit_menu_item = new Applet.MenuItem(_("Edit menu"), Gtk.STOCK_EDIT, Lang.bind(this, this._launch_editor));
-            this._applet_context_menu.addMenuItem(this.edit_menu_item);
-            let settings_menu_item = new Applet.MenuItem(_("Menu settings"), null, function() {
-                Util.spawnCommandLine("cinnamon-settings menu");
-            });
-            this._applet_context_menu.addMenuItem(settings_menu_item);
-
             this._fileFolderAccessActive = false;
 
             this._pathCompleter = new Gio.FilenameCompleter();
             this._pathCompleter.set_dirs_only(false);
             this.lastAcResults = new Array();
 
-            this.searchFilesystem = global.settings.get_boolean("menu-search-filesystem");
-
-            global.settings.connect("changed::menu-search-filesystem", Lang.bind(this, function() {
-                this.searchFilesystem = global.settings.get_boolean("menu-search-filesystem");
-            }));
+            this.settings.bindProperty(Settings.BindingDirection.IN, "search-filesystem", "searchFilesystem", null, null);
         }
         catch (e) {
             global.logError(e);
@@ -886,6 +895,20 @@ MyApplet.prototype = {
         this.menu.open(false);
     },
 
+    _updateActivateOnHover: function() {
+        if (this._openMenuId) {
+            this.actor.disconnect(this._openMenuId);
+            this._openMenuId = 0;
+        }
+        if (this.activateOnHover) {
+            this._openMenuId = this.actor.connect('enter-event', Lang.bind(this, this.openMenu));
+        }
+    },
+
+    _update_hover_delay: function() {
+        this.hover_delay = this.hover_delay_ms / 1000
+    },
+
     on_orientation_changed: function (orientation) {
         this.menu.destroy();
         this.menu = new Applet.AppletPopupMenu(this, orientation);
@@ -900,7 +923,7 @@ MyApplet.prototype = {
         Util.spawnCommandLine("cinnamon-menu-editor");
     },
     
-    on_applet_clicked: function(event) {
+    on_applet_clicked: function(event) {        
         this.menu.toggle_with_options(false);
     },        
            
@@ -978,13 +1001,15 @@ MyApplet.prototype = {
         this.actor.destroy();
         this.emit('destroy');
     },
-    
-    _updateIcon: function(){
-        let icon_file = global.settings.get_string("menu-icon");
-        try{
-           this.set_applet_icon_path(icon_file);               
-        }catch(e){
-           global.logWarning("Could not load icon file \""+icon_file+"\" for menu button");
+
+    _updateIconAndLabel: function(){
+
+        this.set_applet_label(this.menuLabel);
+
+        try {
+           this.set_applet_icon_path(this.menuIcon);
+        } catch(e) {
+           global.logWarning("Could not load icon file \""+this.menuIcon+"\" for menu button");
         }
     },
 
@@ -998,6 +1023,14 @@ MyApplet.prototype = {
         let keyCode = event.get_key_code();
         let modifierState = Cinnamon.get_event_state(event);
 
+        /* check for a keybinding and quit early, otherwise we get a double hit
+           of the keybinding callback */
+        let action = global.display.get_keybinding_action(keyCode, modifierState);
+
+        if (action == Meta.KeyBindingAction.CUSTOM) {
+            return true;
+        }
+
         if (global.display.get_is_overlay_key(keyCode, modifierState) && this.menu.isOpen) {
             this.menu.close();
             return true;
@@ -1313,10 +1346,12 @@ MyApplet.prototype = {
                 this._addEnterEvent(button, Lang.bind(this, function() {
                         this._clearPrevAppSelection(button.actor);
                         button.actor.style_class = "menu-application-button-selected";
+                        this.selectedAppTitle.set_text("");
                         this.selectedAppDescription.set_text(button.place.id.slice(16));
                         }));
                 button.actor.connect('leave-event', Lang.bind(this, function() {
                             this._previousSelectedActor = button.actor;
+                            this.selectedAppTitle.set_text("");
                             this.selectedAppDescription.set_text("");
                             }));
                 this._placesButtons.push(button);
@@ -1366,6 +1401,7 @@ MyApplet.prototype = {
                 this._addEnterEvent(button, Lang.bind(this, function() {
                         this._clearPrevAppSelection(button.actor);
                         button.actor.style_class = "menu-application-button-selected";
+                        this.selectedAppTitle.set_text("");
                         this.selectedAppDescription.set_text(button.file.uri.slice(7));
                         }));
                 button.actor.connect('leave-event', Lang.bind(this, function() {
@@ -1499,7 +1535,6 @@ MyApplet.prototype = {
 
         for (let i = 0; i < this._applicationsButtons.length; i++) {
             this.applicationsBox.add_actor(this._applicationsButtons[i].actor);
-            this._applicationsButtons[i].actor.realize();
             this.applicationsBox.add_actor(this._applicationsButtons[i].menu.actor);
         }
 
@@ -1720,11 +1755,13 @@ MyApplet.prototype = {
         this.categoriesBox = new St.BoxLayout({ style_class: 'menu-categories-box', vertical: true });
         this.applicationsScrollBox = new St.ScrollView({ x_fill: true, y_fill: false, y_align: St.Align.START, style_class: 'vfade menu-applications-scrollbox' });
 
-        this.a11y_settings = new Gio.Settings({ schema: "org.gnome.desktop.a11y.applications" });
+        this.a11y_settings = new Gio.Settings({ schema: "org.cinnamon.desktop.a11y.applications" });
         this.a11y_settings.connect("changed::screen-magnifier-enabled", Lang.bind(this, this._updateVFade));
         this._updateVFade();
-        global.settings.connect("changed::menu-enable-autoscroll", Lang.bind(this, this._update_autoscroll));
+
+        this.settings.bindProperty(Settings.BindingDirection.IN, "enable-autoscroll", "autoscroll_enabled", this._update_autoscroll, null);
         this._update_autoscroll();
+
         let vscroll = this.applicationsScrollBox.get_vscroll_bar();
         vscroll.connect('scroll-start',
                         Lang.bind(this, function() {
@@ -1777,8 +1814,7 @@ MyApplet.prototype = {
     },
 
     _update_autoscroll: function() {
-        let enabled = global.settings.get_boolean("menu-enable-autoscroll");
-        this.applicationsScrollBox.set_auto_scrolling(enabled);
+        this.applicationsScrollBox.set_auto_scrolling(this.autoscroll_enabled);
     },
 
     _clearAllSelections: function(hide_apps) {
@@ -2134,7 +2170,7 @@ MyApplet.prototype = {
     }
 };
 
-function main(metadata, orientation, panel_height) {  
-    let myApplet = new MyApplet(orientation, panel_height);
+function main(metadata, orientation, panel_height, instance_id) {  
+    let myApplet = new MyApplet(orientation, panel_height, instance_id);
     return myApplet;      
 }
