@@ -11,6 +11,7 @@ const GLib = imports.gi.GLib;
 const Gvc = imports.gi.Gvc;
 const Pango = imports.gi.Pango;
 const Tooltips = imports.ui.tooltips;
+const Main = imports.ui.main;
 
 const PropIFace = {
     name: 'org.freedesktop.DBus.Properties',
@@ -99,8 +100,16 @@ const MediaServer2PlayerIFace = {
 
 /* global values */
 let icon_path = "/usr/share/cinnamon/theme/";
-let compatible_players = [ "clementine", "mpd", "exaile", "banshee", "rhythmbox", "rhythmbox3", "pragha", "quodlibet", "guayadeque", "amarok", "googlemusicframe", "xbmc", "noise", "xnoise", "gmusicbrowser", "spotify", "audacious", "vlc", "beatbox", "songbird", "pithos", "gnome-mplayer", "nuvolaplayer", "qmmp" ];
-let support_seek = [ "clementine", "banshee", "rhythmbox", "rhythmbox3", "pragha", "quodlibet", "amarok", "noise", "xnoise", "gmusicbrowser", "spotify", "vlc", "beatbox", "gnome-mplayer", "qmmp" ];
+let compatible_players = [
+    'clementine', 'mpd', 'exaile', 'banshee', 'rhythmbox', 'rhythmbox3',
+    'pragha', 'quodlibet', 'guayadeque', 'amarok', 'googlemusicframe', 'xbmc',
+    'noise', 'xnoise', 'gmusicbrowser', 'spotify', 'audacious', 'vlc',
+    'beatbox', 'songbird', 'pithos', 'gnome-mplayer', 'nuvolaplayer', 'qmmp',
+    'deadbeef', 'smplayer', 'tomahawk'];
+let support_seek = [
+    'clementine', 'banshee', 'rhythmbox', 'rhythmbox3', 'pragha', 'quodlibet',
+    'amarok', 'xnoise', 'gmusicbrowser', 'spotify', 'vlc', 'beatbox',
+    'gnome-mplayer', 'qmmp', 'deadbeef'];
 /* dummy vars for translation */
 let x = _("Playing");
 x = _("Paused");
@@ -283,7 +292,7 @@ ControlButton.prototype = {
             style_class: 'sound-button-icon',
         });
         this.button.set_child(this.icon);
-        this.actor.add_actor(this.button);        
+        this.actor.add_actor(this.button);
     },
     getActor: function() {
         return this.actor;
@@ -458,7 +467,7 @@ Player.prototype = {
         if (support_seek.indexOf(this._name) == -1) {
             this._time.hide();
             this.showPosition = false;
-            this._positionSlider.hide();
+            this._positionSlider.actor.hide();
         }
         this._getStatus();
         this._trackId = {};
@@ -479,7 +488,7 @@ Player.prototype = {
                     this._setStatus(iface, value["playbackStatus"]);
                 if (value["metadata"])
                     this._setMetadata(sender, value["metadata"]);
-            } 
+            }
         }));
 
         this._mediaServerPlayer.connect('Seeked', Lang.bind(this, function(sender, value) {
@@ -515,7 +524,7 @@ Player.prototype = {
     _updatePositionSlider: function(position) {
         this._mediaServerPlayer.getCanSeek(Lang.bind(this, function(sender, canSeek) {
             this._canSeek = canSeek;
-            
+
             if (this._songLength == 0 || position == false)
                 this._canSeek = false
 
@@ -572,7 +581,7 @@ Player.prototype = {
             this._title.setLabel(metadata["xesam:title"].toString());
         else
             this._title.setLabel(_("Unknown Title"));
-        
+
         if (metadata["mpris:trackid"]) {
             this._trackObj = metadata["mpris:trackid"];
         }
@@ -809,6 +818,8 @@ MyApplet.prototype = {
 
             this._control = new Gvc.MixerControl({ name: 'Cinnamon Volume Control' });
             this._control.connect('state-changed', Lang.bind(this, this._onControlStateChanged));
+            this._control.connect('card-added', Lang.bind(this, this._onControlStateChanged));
+            this._control.connect('card-removed', Lang.bind(this, this._onControlStateChanged));
             this._control.connect('default-sink-changed', Lang.bind(this, this._readOutput));
             this._control.connect('default-source-changed', Lang.bind(this, this._readInput));
             this._control.connect('stream-added', Lang.bind(this, this._maybeShowInput));
@@ -1058,9 +1069,8 @@ MyApplet.prototype = {
         this[property].push_volume();
     },
 
-    _notifyVolumeChange: function() {
-        global.cancel_theme_sound(VOLUME_NOTIFY_ID);
-        global.play_theme_sound(VOLUME_NOTIFY_ID, 'audio-volume-change');
+    _notifyVolumeChange: function() {        
+        Main.soundManager.play('volume');
     },
 
     _mutedChanged: function(object, param_spec, property) {
