@@ -210,7 +210,7 @@ ClassicSwitcher.prototype = {
     },
 
     _onWorkspaceSelected: function() {
-        this._windows = this._getWindowsForBinding(this._binding);
+        this._windows = AppSwitcher.getWindowsForBinding(this._binding);
         this._currentIndex = 0;
         this._updateList(0);
         this._select(0);
@@ -407,7 +407,22 @@ AppIcon.prototype = {
         this.actor.add(this._iconBin, { x_fill: false, y_fill: false } );
         let title = window.get_title();
         if (title) {
-            this.label = new St.Label({ text: title });
+            if (window.minimized) {
+                this.label = new St.Label({ text: "[" + title + "]"});               
+                let contrast_effect = new Clutter.BrightnessContrastEffect();                
+                contrast_effect.set_brightness(-0.5, -0.5, -0.5);                         
+                this._iconBin.add_effect(contrast_effect);                
+            }
+            else if (window.tile_type == Meta.WindowTileType.TILED) {
+                this.label = new St.Label({ text: "|" + title });
+            }
+            else if (window.tile_type == Meta.WindowTileType.SNAPPED) {
+                this.label = new St.Label({ text: "||" + title });
+            }
+            else {
+                this.label = new St.Label({ text: title });    
+            }
+            
             let bin = new St.Bin({ x_align: St.Align.MIDDLE });
             bin.add_actor(this.label);
             this.actor.add(bin);
@@ -422,7 +437,7 @@ AppIcon.prototype = {
         if (this.showThumbnail){
             this.icon = new St.Group();
             let clones = WindowUtils.createWindowClone(this.window, size, size, true, true);
-            for (i in clones) {
+            for (let i in clones) {
                 let clone = clones[i];
                 this.icon.add_actor(clone.actor);
                 // the following 2 lines are used when cloning without positions (param #4 = false)
