@@ -441,12 +441,11 @@ Notification.prototype = {
         this._titleFitsInBannerMode = true;
         this._titleDirection = St.TextDirection.NONE;
         this._spacing = 0;
-        this._scrollPolicy = Gtk.PolicyType.AUTOMATIC;
+
         this._imageBin = null;
         this._timestamp = new Date();
         this._inNotificationBin = false;
-        let calendarSettings = new Gio.Settings({ schema: 'org.cinnamon.calendar' });
-        this.dateFormat = calendarSettings.get_string('date-format');
+        this.dateFormat = _("%l:%M %p");
 
         source.connect('destroy', Lang.bind(this,
             function (source, reason) {
@@ -607,16 +606,10 @@ Notification.prototype = {
         this._icon.visible = visible;
     },
 
-    enableScrolling: function(enableScrolling) {
-        this._scrollPolicy = enableScrolling ? Gtk.PolicyType.AUTOMATIC : Gtk.PolicyType.NEVER;
-        if (this._scrollArea)
-            this._scrollArea.vscrollbar_policy = this._scrollPolicy;
-    },
-
     _createScrollArea: function() {
         this._table.add_style_class_name('multi-line-notification');
         this._scrollArea = new St.ScrollView({ name: 'notification-scrollview',
-                                               vscrollbar_policy: this._scrollPolicy,
+                                               vscrollbar_policy: Gtk.PolicyType.NEVER,
                                                hscrollbar_policy: Gtk.PolicyType.NEVER,
                                                style_class: 'vfade' });
         this._table.add(this._scrollArea, { row: 1,
@@ -759,11 +752,11 @@ Notification.prototype = {
         if (!this._buttonBox) {
 
             let box = new St.BoxLayout({ name: 'notification-actions' });
-            this.setActionArea(box, { x_expand: false,
+            this.setActionArea(box, { x_expand: true,
                                       y_expand: false,
-                                      x_fill: false,
+                                      x_fill: true,
                                       y_fill: false,
-                                      x_align: St.Align.END });
+                                      x_align: St.Align.START });
             this._buttonBox = box;
         }
 
@@ -1318,7 +1311,6 @@ SummaryItem.prototype = {
             if (notification.actor.get_parent() == this.notificationStack)
                 this.notificationStack.remove_actor(notification.actor);
             notification.setIconVisible(true);
-            notification.enableScrolling(true);
         }
         this._stackedNotifications = [];
     },
@@ -1335,8 +1327,6 @@ SummaryItem.prototype = {
         stackedNotification.notificationDoneDisplayingId = notification.connect('done-displaying', Lang.bind(this, this._notificationDoneDisplaying));
         stackedNotification.notificationDestroyedId = notification.connect('destroy', Lang.bind(this, this._notificationDestroyed));
         this._stackedNotifications.push(stackedNotification);
-        if (!this.source.isChat)
-            notification.enableScrolling(false);
         if (this.notificationStack.get_children().length > 0)
             notification.setIconVisible(false);
         this.notificationStack.add(notification.actor);
@@ -1669,15 +1659,7 @@ MessageTray.prototype = {
     _updateShowingNotification: function() {
         Tweener.removeTweens(this._notificationBin);
 
-        // We auto-expand notifications with CRITICAL urgency.
-        // We use Tweener.removeTweens() to remove a tween that was hiding the notification we are
-        // updating, in case that notification was in the process of being hidden. However,
-        // Tweener.removeTweens() would also remove a tween that was updating the position of the
-        // notification we are updating, in case that notification was already expanded and its height
-        // changed. Therefore we need to call this._expandNotification() for expanded notifications
-        // to make sure their position is updated.
-        if (this._notification.urgency == Urgency.CRITICAL || this._notification.expanded)
-            this._expandNotification(true);		
+        this._expandNotification(true);
 
         // We tween all notifications to full opacity. This ensures that both new notifications and
         // notifications that might have been in the process of hiding get full opacity.
